@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2011-2013 Yubico AB
+ * Copyright (c) 2011-2014 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,6 +36,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 
+#include <yubikey.h>
 #include <ykpers.h>
 #include <ykdef.h>
 /*
@@ -54,9 +55,9 @@ struct ykp_config_t {
 	struct config_st *ykcore_config;
 };
 
-void _yktest_hexdump(char *prefix, void *buffer, int size, int break_on)
+static void _yktest_hexdump(const char *prefix, const void *buffer, int size, int break_on)
 {
-	unsigned char *p = buffer;
+	unsigned const char *p = buffer;
 	int i;
 	if (prefix != NULL)
 		fprintf(stderr, "%s", prefix);
@@ -73,7 +74,7 @@ void _yktest_hexdump(char *prefix, void *buffer, int size, int break_on)
 	fflush(stderr);
 }
 
-void _check_success(int rc, YKP_CONFIG *cfg, unsigned char expected[], int caller_line)
+static void _check_success(int rc, YKP_CONFIG *cfg, unsigned char expected[], int caller_line)
 {
 	struct config_st *ycfg;
 	bool config_matches_expected = false;
@@ -99,13 +100,13 @@ void _check_success(int rc, YKP_CONFIG *cfg, unsigned char expected[], int calle
 	assert(config_matches_expected == true);
 }
 
-int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, char **argv)
+static int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, const char **argv)
 {
 	const char *infname = NULL;
 	const char *outfname = NULL;
 	bool verbose = false;
 	bool dry_run = false;
-	bool aesviahash = false;
+	char keylocation = 0;
 	bool use_access_code = false;
 	unsigned char access_code[256];
 	YK_KEY *yk = 0;
@@ -114,10 +115,12 @@ int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, char **argv)
 	int data_format = YKP_FORMAT_LEGACY;
 
 	/* Options */
-	char *salt = NULL;
 	char ndef[128];
 	char ndef_type = 0;
-	unsigned char usb_mode = -1;
+	unsigned char usb_mode = 0;
+	unsigned char cr_timeout = 0;
+	unsigned char autoeject_timeout = 0;
+	int num_modes_seen = 0;
 	bool zap = false;
 
 	unsigned char scan_map[sizeof(SCAN_MAP)];
@@ -139,17 +142,17 @@ int _test_config (YKP_CONFIG *cfg, YK_STATUS *st, int argc, char **argv)
 	/* call args_to_config from ykpers-args.c with a fake set of program arguments */
 	rc = args_to_config(argc, argv, cfg, yk,
 			    &infname, &outfname,
-			    &data_format,
-			    &autocommit, salt,
+			    &data_format, &autocommit,
 			    st, &verbose, &dry_run,
 			    access_code, &use_access_code,
-			    &aesviahash, &ndef_type, ndef, &usb_mode, &zap,
-			    scan_map, &exit_code);
+			    &keylocation, &ndef_type, ndef, &usb_mode, &zap,
+			    scan_map, &cr_timeout, &autoeject_timeout, &num_modes_seen,
+			    &exit_code);
 
 	return rc;
 }
 
-YK_STATUS * _test_init_st(int major, int minor, int build)
+static YK_STATUS * _test_init_st(int major, int minor, int build)
 {
 	YK_STATUS *st = ykds_alloc();
 	struct status_st *t;
@@ -168,7 +171,7 @@ YK_STATUS * _test_init_st(int major, int minor, int build)
  * Utility function to parse arguments and just return the result code.
  * The calling function does the assert() to get function name in assert output.
  */
-int _parse_args_rc(int argc, char *argv[])
+static int _parse_args_rc(int argc, const char *argv[])
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
@@ -182,12 +185,11 @@ int _parse_args_rc(int argc, char *argv[])
 	return rc;
 }
 
-int _test_config_slot1(void)
+static void _test_config_slot1(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(1, 3, 0);
 	int rc = 0;
-	struct config_st *ycfg;
 
 	unsigned char expected[] = {
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -200,7 +202,7 @@ int _test_config_slot1(void)
 		0x00, 0x46, 0xc0
 	};
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1",
 		NULL
 	};
@@ -213,12 +215,11 @@ int _test_config_slot1(void)
 	free(st);
 }
 
-int _test_config_static_slot2(void)
+static void _test_config_static_slot2(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 0, 0);
 	int rc = 0;
-	struct config_st *ycfg;
 
 	unsigned char expected[] = {
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -231,7 +232,7 @@ int _test_config_static_slot2(void)
 		0x00, 0xe9, 0xf5
 	};
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-2", "-a303132333435363738393a3b3c3d3e3f",
 		NULL
 	};
@@ -244,13 +245,13 @@ int _test_config_static_slot2(void)
 	free(st);
 }
 
-int _test_too_old_key(void)
+static void _test_too_old_key(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(1, 3, 0);
 	int rc = 0;
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-oshort-ticket",
 		NULL
 	};
@@ -264,13 +265,13 @@ int _test_too_old_key(void)
 	free(st);
 }
 
-int _test_too_new_key(void)
+static void _test_too_new_key(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
 	int rc = 0;
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-oticket-first",
 		NULL
 	};
@@ -284,7 +285,7 @@ int _test_too_new_key(void)
 	free(st);
 }
 
-int _test_non_config_args(void)
+static void _test_non_config_args(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
@@ -294,7 +295,7 @@ int _test_non_config_args(void)
 	const char *outfname = NULL;
 	bool verbose = false;
 	bool dry_run = false;
-	bool aesviahash = false;
+	char keylocation = 0;
 	bool use_access_code = false;
 	unsigned char access_code[256];
 	YK_KEY *yk = 0;
@@ -304,15 +305,17 @@ int _test_non_config_args(void)
 	int data_format = YKP_FORMAT_LEGACY;
 
 	/* Options */
-	char *salt = NULL;
 	char ndef[128];
 	char ndef_type = 0;
-	unsigned char usb_mode = -1;
+	unsigned char usb_mode = 0;
+	unsigned char cr_timeout = 0;
+	unsigned char autoeject_timeout = 0;
+	int num_modes_seen = 0;
 	bool zap = false;
 
 	unsigned char scan_map[sizeof(SCAN_MAP)];
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1", "-sout", "-iin", "-c313233343536", "-y", "-v",
 		NULL
 	};
@@ -334,16 +337,16 @@ int _test_non_config_args(void)
 	/* call args_to_config from ykpers-args.c with a fake set of program arguments */
 	rc = args_to_config(argc, argv, cfg, yk,
 			    &infname, &outfname,
-			    &data_format,
-			    &autocommit, salt,
+			    &data_format, &autocommit,
 			    st, &verbose, &dry_run,
 			    access_code, &use_access_code,
-			    &aesviahash, &ndef_type, ndef, &usb_mode, &zap,
-			    scan_map, &exit_code);
+			    &keylocation, &ndef_type, ndef, &usb_mode, &zap,
+			    scan_map, &cr_timeout, &autoeject_timeout, &num_modes_seen,
+			    &exit_code);
 	assert(rc == 1);
 	i = strcmp(infname, "in"); assert(i == 0);
 	i = strcmp(outfname, "out"); assert(i == 0);
-	i = strcmp(access_code, "123456"); assert(i == 0);
+	i = memcmp(access_code, "123456", 6); assert(i == 0);
 	assert(autocommit == true);
 	assert(verbose == true);
 
@@ -351,12 +354,11 @@ int _test_non_config_args(void)
 	free(st);
 }
 
-int _test_oath_hotp_nist_160_bits(void)
+static void _test_oath_hotp_nist_160_bits(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 1, 0);
 	int rc = 0;
-	struct config_st *ycfg;
 
 	unsigned char expected[] = {
 		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -369,7 +371,7 @@ int _test_oath_hotp_nist_160_bits(void)
 		0x00, 0x6a, 0xb9
 	};
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1", "-a303132333435363738393a3b3c3d3e3f40414243", "-ooath-hotp", "-o-append-cr",
 		NULL
 	};
@@ -382,7 +384,7 @@ int _test_oath_hotp_nist_160_bits(void)
 	free(st);
 }
 
-int _test_extended_flags1(void)
+static void _test_extended_flags1(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 0);
@@ -402,7 +404,7 @@ int _test_extended_flags1(void)
 		0x00, 0x03, 0x95, 0x56, 0x00, 0x00, 0x00,
 	};
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-2", "-a303132333435363738393a3b3c3d3e3f40414243",
 		"-ochal-resp", "-ochal-hmac", "-ohmac-lt64", "-oserial-api-visible",
 		NULL
@@ -416,10 +418,10 @@ int _test_extended_flags1(void)
 	free(st);
 }
 
-int _test_two_slots1(void)
+static void _test_two_slots1(void)
 {
 	/* Test that it is not possible to choose slot more than once */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1", "-1",
 		NULL
 	};
@@ -427,10 +429,10 @@ int _test_two_slots1(void)
 	assert(rc == 0);
 }
 
-int _test_two_slots2(void)
+static void _test_two_slots2(void)
 {
 	/* Test that it is not possible to choose slot more than once */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-2", "-1",
 		NULL
 	};
@@ -438,10 +440,10 @@ int _test_two_slots2(void)
 	assert(rc == 0);
 }
 
-int _test_two_modes_at_once1(void)
+static void _test_two_modes_at_once1(void)
 {
 	/* Test that it is not possible to choose mode (OATH-HOTP/CHAL-RESP) more than once */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-ochal-resp", "-ooath-hotp",
 		NULL
 	};
@@ -449,10 +451,10 @@ int _test_two_modes_at_once1(void)
 	assert(rc == 0);
 }
 
-int _test_two_modes_at_once2(void)
+static void _test_two_modes_at_once2(void)
 {
 	/* Test that it is not possible to choose mode (OATH-HOTP/CHAL-RESP) more than once */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-ochal-resp", "-ochal-resp",
 		NULL
 	};
@@ -460,10 +462,10 @@ int _test_two_modes_at_once2(void)
 	assert(rc == 0);
 }
 
-int _test_mode_after_other_option(void)
+static void _test_mode_after_other_option(void)
 {
 	/* Test that it is not possible to set mode after other options */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-ohmac-lt64", "-ochal-resp",
 		NULL
 	};
@@ -471,12 +473,12 @@ int _test_mode_after_other_option(void)
 	assert(rc == 0);
 }
 
-int _test_key_mixed_case1(void)
+static void _test_key_mixed_case1(void)
 {
 	/* Make sure key with mixed case is rejected (parsing function yubikey_hex_decode
 	 * only handles lower case hex)
 	 */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1", "-a0000000000000000000000000000000E",
 		NULL
 	};
@@ -484,10 +486,10 @@ int _test_key_mixed_case1(void)
 	assert(rc == 0);
 }
 
-int _test_uid_for_oath(void)
+static void _test_uid_for_oath(void)
 {
 	/* Test that it is not possible to specify UID with OATH */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-ooath-hotp", "-ouid=h:010203040506",
 		NULL
 	};
@@ -495,10 +497,10 @@ int _test_uid_for_oath(void)
 	assert(rc == 0);
 }
 
-int _test_uid_for_chal_resp(void)
+static void _test_uid_for_chal_resp(void)
 {
 	/* Test that it is not possible to specify UID with Challenge Response */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-ochal-resp", "-ouid=h:010203040506",
 		NULL
 	};
@@ -506,10 +508,10 @@ int _test_uid_for_chal_resp(void)
 	assert(rc == 0);
 }
 
-int _test_swap_with_slot(void)
+static void _test_swap_with_slot(void)
 {
 	/* Test that you can not both swap and set slot */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-x", "-1",
 		NULL
 	};
@@ -517,10 +519,10 @@ int _test_swap_with_slot(void)
 	assert(rc == 0);
 }
 
-int _test_slot_with_update(void)
+static void _test_slot_with_update(void)
 {
 	/* Test the update must be before slot */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-1", "-u",
 		NULL
 	};
@@ -528,10 +530,10 @@ int _test_slot_with_update(void)
 	assert(rc == 0);
 }
 
-int _test_swap_with_update(void)
+static void _test_swap_with_update(void)
 {
 	/* Test the update must be before slot */
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-u", "-x",
 		NULL
 	};
@@ -539,12 +541,12 @@ int _test_swap_with_update(void)
 	assert(rc == 0);
 }
 
-int _test_ndef_for_neo_beta(void)
+static void _test_ndef_for_neo_beta(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 1, 7);
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-nhttps://my.yubico.com/neo/",
 		NULL
 	};
@@ -552,19 +554,18 @@ int _test_ndef_for_neo_beta(void)
 
 	int rc = _test_config(cfg, st, argc, argv);
 	assert(rc == 1);
-	struct config_st *ycfg = (struct config_st *) ykp_core_config(cfg);
 	assert(((struct ykp_config_t*)cfg)->command == SLOT_NDEF);
 
 	ykp_free_config(cfg);
 	free(st);
 }
 
-int _test_ndef_with_non_neo(void)
+static void _test_ndef_with_non_neo(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 2, 4);
 
-	char *argv[] = {
+	const char *argv[] = {
 		"unittest", "-nhttps://my.yubico.com/neo/",
 		NULL
 	};
@@ -577,12 +578,12 @@ int _test_ndef_with_non_neo(void)
 	free(st);
 }
 
-int _test_slot_two_with_neo_beta(void)
+static void _test_slot_two_with_neo_beta(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 1, 7);
 
-	char *argv[] = {
+	const const char *argv[] = {
 		"unittest", "-2", NULL
 	};
 	int argc = 2;
@@ -593,15 +594,16 @@ int _test_slot_two_with_neo_beta(void)
 	free(st);
 }
 
-int _test_ndef2_with_neo_beta(void)
+static void _test_ndef2_with_neo_beta(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(2, 1, 7);
 
-	char *argv[] = {
+	const const char *argv[] = {
 		"unittest", "-2", "-nhttps://my.yubico.com/neo/",
+		NULL
 	};
-	int argc = 2;
+	int argc = 3;
 
 	int rc = _test_config(cfg, st, argc, argv);
 	assert(rc == 0);
@@ -609,12 +611,12 @@ int _test_ndef2_with_neo_beta(void)
 	free(st);
 }
 
-int _test_ndef2_with_neo(void)
+static void _test_ndef2_with_neo(void)
 {
 	YKP_CONFIG *cfg = ykp_alloc();
 	YK_STATUS *st = _test_init_st(3, 0, 0);
 
-	char *argv[] = {
+	const const char *argv[] = {
 		"unittest", "-2", "-nhttps://my.yubico.com/neo/",
 		NULL
 	};
@@ -622,14 +624,13 @@ int _test_ndef2_with_neo(void)
 
 	int rc = _test_config(cfg, st, argc, argv);
 	assert(rc == 1);
-	struct config_st *ycfg = (struct config_st *) ykp_core_config(cfg);
 	assert(((struct ykp_config_t*)cfg)->command == SLOT_NDEF2);
 
 	ykp_free_config(cfg);
 	free(st);
 }
 
-int main (int argc, char **argv)
+int main (void)
 {
 	_test_config_slot1();
 	_test_config_static_slot2();
