@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008-2012 Yubico AB
+ * Copyright (c) 2008-2014 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -147,6 +147,10 @@ const char *yk_strerror(int errnum);
    error.  */
 const char *yk_usb_strerror(void);
 
+
+/* Swaps the two bytes between little and big endian on big endian machines */
+extern uint16_t yk_endian_swap_16(uint16_t x);
+
 #define YK_EUSBERR	0x01	/* USB error reporting should be used */
 #define YK_EWRONGSIZ	0x02
 #define YK_EWRITEERR	0x03
@@ -160,6 +164,7 @@ const char *yk_usb_strerror(void);
 #define YK_EWOULDBLOCK	0x0b	/* operation would block */
 #define YK_EINVALIDCMD	0x0c	/* supplied command is invalid for this operation */
 #define YK_EMORETHANONE	0x0d    /* expected to find only one key but found more */
+#define YK_ENODATA	0x0e	/* no data was returned from a read */
 
 /* Flags for response reading. Use high numbers to not exclude the possibility
  * to combine these with for example SLOT commands from ykdef.h in the future.
